/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.common.dictionary;

import java.util.ArrayList;

import org.vono.narau.R;
import org.vono.narau.models.dictionary.Entry;
import org.vono.narau.preferences.Preferences;

import android.content.Context;
import android.os.Parcel;
import android.os.Parcelable;
import android.text.Html;
import android.text.method.LinkMovementMethod;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.TextView;

/**
 * Entry ArrayAdapter for DictionaryActivity and KanjiToWords.
 * 
 */
public class EntryArrayAdapter extends ArrayAdapter<Entry> implements Parcelable {

	private final int colorBlack;
	private final int colorDarkGrey;
	int positionWithDetail;

	public EntryArrayAdapter(Context context) {
		super(context, R.layout.dictionary_offline_item);
		this.colorBlack = context.getResources().getColor(android.R.color.black);
		this.colorDarkGrey = context.getResources().getColor(R.color.dark_grey);
		this.positionWithDetail = -1;
	}

	@Override
	public long getItemId(int position) {
		return super.getItem(position).sequence;
	}

	@Override
	public boolean hasStableIds() {
		return true;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		LinearLayout ill = (LinearLayout) convertView;
		final float lZoomValue = Preferences.getZoom();

		if (null == ill) {
			final LayoutInflater inflator = (LayoutInflater) super.getContext().getSystemService(
					Context.LAYOUT_INFLATER_SERVICE);
			ill = (LinearLayout) inflator.inflate(R.layout.dictionary_offline_item, null);
		}

		ill.setBackgroundColor(position % 2 == 0 ? this.colorDarkGrey : this.colorBlack);

		final TextView tvKanji = (TextView) ill.findViewById(R.id.dictionaryEntryTextViewKanji);
		final TextView tvReading = (TextView) ill.findViewById(R.id.dictionaryEntryTextViewReading);
		final TextView tvSense = (TextView) ill.findViewById(R.id.dictionaryEntryTextViewSense);
		tvKanji.setTextSize(TypedValue.COMPLEX_UNIT_PX, lZoomValue);
		tvReading.setTextSize(TypedValue.COMPLEX_UNIT_PX, lZoomValue);
		tvSense.setTextSize(TypedValue.COMPLEX_UNIT_PX, lZoomValue);

		final Entry entry = super.getItem(position);

		if (this.positionWithDetail == position) {
			entry.loadFull();

			tvKanji.setText(Html.fromHtml(entry.getHTMLKanjiElems()));
			tvKanji.setMovementMethod(LinkMovementMethod.getInstance());

			tvReading.setText(Html.fromHtml(entry.getHTMLReadingElems()));
			tvReading.setMovementMethod(LinkMovementMethod.getInstance());

			tvSense.setText(Html.fromHtml(entry.getHTMLSenses()));
			LayoutParams lp = new LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
					ViewGroup.LayoutParams.WRAP_CONTENT, 1.0f);
			tvSense.setLayoutParams(lp);
			tvSense.setMovementMethod(LinkMovementMethod.getInstance());
			// Try to put the tvSense displayed
			// tvSense.requestFocusFromTouch();
		} else {
			tvKanji.setText(entry.getFirstKanjiElement());
			tvReading.setText(entry.getFirstReadingElement());
			tvSense.setText(entry.getFirstGloss());

			tvKanji.setMovementMethod(null);
			tvReading.setMovementMethod(null);
			tvSense.setMovementMethod(null);
		}

		return ill;
	}

	public void setEntries(final ArrayList<Entry> items) {
		this.clear();

		super.setNotifyOnChange(false);
		final int count = items.size();
		for (int i = 0; i < count; i++) {
			super.add(items.get(i));
		}
		super.setNotifyOnChange(true);

		super.notifyDataSetChanged();
	}

	public void setShowHideDetail(int position) {
		if (this.positionWithDetail == position) {
			this.positionWithDetail = -1;
		} else {
			this.positionWithDetail = position;
		}
		super.notifyDataSetChanged();
	}

	@Override
	public void clear() {
		this.positionWithDetail = -1;
		super.clear();
	}

	public int describeContents() {
		return 0;
	}

	public void writeToParcel(Parcel dest, int flags) {
		// Nothing to do, not used in my case by android
	}

}
