/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.common.dictionary;

import org.vono.narau.R;

import android.app.Activity;
import android.content.res.TypedArray;
import android.os.Bundle;
import android.support.v4.app.ListFragment;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewGroup.LayoutParams;
import android.widget.ListView;

public class EntryListFragment extends ListFragment {
	public static final String TAG = EntryListFragment.class.getSimpleName();
	private static final String KEY_LIST_ENTRIES = "LE";

	private EntryArrayAdapter adapter;
	private int headerLayoutId;

	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		this.adapter = new EntryArrayAdapter(activity);
	}

	@Override
	public void onInflate(Activity activity, AttributeSet attrs, Bundle savedInstanceState) {
		super.onInflate(activity, attrs, savedInstanceState);

		// From Sample ApiDemos/src/com/example/android/apis/app/FragmentArguments.java
		TypedArray ta = activity.obtainStyledAttributes(attrs, R.styleable.FragmentArguments);
		this.headerLayoutId = ta.getResourceId(R.styleable.FragmentArguments_header, 0);
		ta.recycle();
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		if (null != savedInstanceState) {
			this.adapter = savedInstanceState.getParcelable(KEY_LIST_ENTRIES);
			this.adapter.notifyDataSetChanged();
		}

		super.setListAdapter(this.adapter);

	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		View v = super.onCreateView(inflater, container, savedInstanceState);

		if (0 != this.headerLayoutId) {
			View header = super.getLayoutInflater(savedInstanceState).inflate(this.headerLayoutId, null);

			// http://stackoverflow.com/a/4401369
			LayoutParams lp = new ListView.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT);
			header.setLayoutParams(lp);
			ListView lv = (ListView) v.findViewById(android.R.id.list);
			lv.addHeaderView(header);
		}

		return v;

	}

	@Override
	public void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);
		outState.putParcelable(KEY_LIST_ENTRIES, this.adapter);
	}

	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		this.adapter.setShowHideDetail(position - super.getListView().getHeaderViewsCount());
	}

	public void clear() {
		this.adapter.clear();
	}

}
