/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.dictionary;

import java.text.DateFormat;
import java.util.ArrayList;

import org.vono.narau.R;
import org.vono.narau.db.HistoryDB;
import org.vono.narau.models.history.SearchResult;
import org.vono.narau.preferences.Preferences;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.ListFragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

public class HistoryFragment extends ListFragment {

	static final String KEY_DICTIONARY_TYPE = "KEY_DICT_TYPE";

	private OnSearchSelectedListener mListener;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	}

	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);

		this.mListener = (OnSearchSelectedListener) activity;
	}

	@Override
	public void onResume() {
		super.onResume();

		final SearchAdapter searchAdapter = new SearchAdapter(super.getActivity(), this.mListener.getDictionaryType());
		setListAdapter(searchAdapter);
	}

	@Override
	public void onPause() {
		super.onPause();
	}

	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		final SearchResult sr = (SearchResult) super.getListAdapter().getItem(position);
		this.mListener.onHistorySearchSelected(sr);
	}

	public void addSearch(SearchResult sr) {
		((SearchAdapter) super.getListAdapter()).add(sr);
	}

	public void setTextSize(float size) {
		final SearchAdapter searchAdapter = ((SearchAdapter) super.getListAdapter());
		searchAdapter.textSize = size;
		searchAdapter.notifyDataSetChanged();
	}

	/**
	 * The search array adapter.
	 * 
	 */
	public static class SearchAdapter extends ArrayAdapter<SearchResult> {
		private LayoutInflater mInflater;
		private float textSize;

		public SearchAdapter(Context context, DictionaryType type) {
			super(context, R.layout.dictionary_search_result_item);
			this.mInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			// if (android.os.Build.VERSION.SDK_INT >=
			// android.os.Build.VERSION_CODES.HONEYCOMB) {
			// super.addAll(HistoryDB.getHistoryDB().getAll());
			// } else {
			//
			// }
			if (DictionaryType.offline == type) {
				this.addArray(HistoryDB.getHistoryDB().getAllOfflineResult());
			} else {
				this.addArray(HistoryDB.getHistoryDB().getAllOnlineResults());
			}

			this.textSize = Preferences.getFloat(R.string.prefNameTextZoom);
		}

		@Override
		public long getItemId(int position) {
			return super.getItem(position).searchTerm.hashCode();
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			final SearchResult sr = super.getItem(position);

			final View view;
			if (null == convertView) {
				view = this.mInflater.inflate(R.layout.dictionary_search_result_item, parent, false);
			} else {
				view = convertView;
			}

			final TextView tvSearch = (TextView) view.findViewById(R.id.dictionarySearchResultText);
			final TextView tvNumResult = (TextView) view.findViewById(R.id.dictionarySearchResultNumResult);
			final TextView tvLang = (TextView) view.findViewById(R.id.dictionarySearchResultLang);
			final TextView tvDate = (TextView) view.findViewById(R.id.dictionarySearchResultDate);

			final DateFormat df = DateFormat.getInstance();

			tvSearch.setText(sr.searchTerm);
			tvNumResult.setText(Integer.toString(sr.numResults));
			tvLang.setText(sr.getDisplayLanguage());
			tvDate.setText(df.format(sr.date));

			tvSearch.setTextSize(this.textSize);
			// tvNumResult.setTextSize(this.textSize);
			// tvLang.setTextSize(this.textSize);
			// tvDate.setTextSize(this.textSize);

			return view;
		}

		@Override
		public void add(SearchResult sr) {
			super.remove(sr);
			super.insert(sr, 0);
		}

		public void addArray(ArrayList<SearchResult> list) {
			final int count = list.size();
			for (int i = 0; i < count; i++) {
				super.add(list.get(i));
			}
		}

	}

	// Container Activity must implement this interface
	public interface OnSearchSelectedListener {
		/**
		 * Callback function to notifie that the user have clicked on this search result.
		 * 
		 * @param sr
		 *            The search result selected
		 */
		public void onHistorySearchSelected(SearchResult sr);

		public DictionaryType getDictionaryType();
	}

}
