/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.learn.reading;

import java.util.ArrayList;
import java.util.Collections;

import org.vono.narau.R;
import org.vono.narau.common.Common;
import org.vono.narau.common.dictionary.EntryArrayAdapter;
import org.vono.narau.common.dictionary.EntryListFragment;
import org.vono.narau.common.kanji.KanjiArrayAdapter;
import org.vono.narau.common.kanji.KanjiListSelectActivity;
import org.vono.narau.db.DatabaseType;
import org.vono.narau.db.DictionaryDB;
import org.vono.narau.db.KanjiDB;
import org.vono.narau.models.dictionary.Entry;
import org.vono.narau.models.kanji.Kanji;
import org.vono.narau.models.kanji.KanjiData;
import org.vono.narau.preferences.Preferences;
import org.vono.narau.utils.Language;

import android.annotation.SuppressLint;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v4.app.FragmentActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

public class KanjiToWordActivity extends FragmentActivity {
	private static final String TAG = KanjiToWordActivity.class.getSimpleName();
	private static final String CURRENT_POSITION = "CP";
	private static final String LOAD_KANJIS_DONE = "lkd";
	private static final String LOAD_KANJIS_SINGLE_DONE = "lksd";
	private static final String LOAD_KANJIS_WAIT = "lkw";
	private static final int ALL_KANJI_CODE = 456;

	static final int TYPE_MASK = KanjiData.KANJI_JLPT | KanjiData.KANJI_GRADE;

	private static ArrayList<KanjiWords> listKanjiWords = null;

	private TextView textKanjiHeader;
	private TextView textKanjiMeaning;
	private ProgressDialog progressDialog;
	private ImageButton buttonNext;
	private ImageButton buttonPrevious;

	private static LoaderThreadKanji loaderThreadKanji;
	private static LoaderThreadWords loaderThreadWords;
	private static LoaderHandler loaderHandler;
	private EntryListFragment entryListFragment;

	private int currentPosition;
	private int defaultLimit;

	public KanjiToWordActivity() {
		this.textKanjiHeader = null;
		this.textKanjiMeaning = null;
		this.progressDialog = null;
		this.entryListFragment = null;

		this.currentPosition = 0;
		this.defaultLimit = Preferences.getInt(R.string.prefNameKanjiToWordsNWTL);

		if (null == KanjiToWordActivity.listKanjiWords) {
			KanjiToWordActivity.listKanjiWords = new ArrayList<KanjiToWordActivity.KanjiWords>(100);
		}

	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Bundle extra = this.getIntent().getExtras();

		if (null == savedInstanceState && null != extra) {

			KanjiData.WritingCategory wc = (KanjiData.WritingCategory) extra
					.getSerializable(KanjiArrayAdapter.ITEM_CATEGORY);

			final ArrayList<KanjiData.CharacterCatalog> alCC = wc.catalogList;
			final int numCC = alCC.size();
			boolean haveCatalogSelected = false;
			StringBuilder sbSelection = new StringBuilder();
			ArrayList<String> selectionArgsList = new ArrayList<String>(numCC);

			for (int i = 0; i < numCC; i++) {
				final KanjiData.CharacterCatalog cc = alCC.get(i);
				if (cc.isSelected() && (cc.typeMask & TYPE_MASK) == cc.typeMask) {
					haveCatalogSelected = true;
					if (sbSelection.length() > 0) {
						sbSelection.append(" or ");
					}
					sbSelection.append(cc.selection);
					selectionArgsList.addAll(cc.selectionArgs);
				}
			}

			if (haveCatalogSelected) {

				this.setContentView();

				this.progressDialog = ProgressDialog.show(this, Common.EMPTY_STRING,
						Common.getString(R.string.loading), true);
				this.currentPosition = 0;

				String selection = sbSelection.toString();
				String[] selectionArgs = selectionArgsList.toArray(new String[0]);

				KanjiToWordActivity.loaderHandler = new LoaderHandler(this);
				KanjiToWordActivity.loaderThreadWords = new LoaderThreadWords();
				KanjiToWordActivity.loaderThreadWords.setPriority(Thread.MIN_PRIORITY);
				KanjiToWordActivity.loaderThreadKanji = new LoaderThreadKanji(selection, selectionArgs);
				KanjiToWordActivity.loaderThreadKanji.setPriority(Thread.MIN_PRIORITY);
				KanjiToWordActivity.loaderThreadKanji.start();

			} else {
				Log.e(TAG, "No kanji to display");
				this.finish();
			}
		}
	}

	@Override
	public void onDestroy() {
		super.onDestroy();

		this.buttonNext = null;
		this.buttonPrevious = null;
		this.textKanjiHeader = null;
		this.textKanjiMeaning = null;

		if (null != KanjiToWordActivity.loaderHandler && this == KanjiToWordActivity.loaderHandler.activity) {
			KanjiToWordActivity.loaderHandler.activity = null;
		}

		System.gc();
		System.runFinalization();
	}

	/**
	 * <p>
	 * Handle device rotation.
	 * </p>
	 * {@inheritDoc}
	 * 
	 */
	@Override
	protected void onSaveInstanceState(Bundle savedInstanceState) {
		super.onSaveInstanceState(savedInstanceState);
		savedInstanceState.putInt(CURRENT_POSITION, this.currentPosition);
	}

	/**
	 * <p>
	 * Handle device rotation.
	 * </p>
	 * {@inheritDoc}
	 * 
	 */
	@Override
	protected void onRestoreInstanceState(Bundle savedInstanceState) {

		this.setContentView();

		KanjiToWordActivity.loaderHandler.activity = this;

		this.currentPosition = savedInstanceState.getInt(CURRENT_POSITION);

		final KanjiWords kanjiWords = KanjiToWordActivity.getKanjiWords(this.currentPosition);
		if (null != kanjiWords && null != kanjiWords.entries) {

			this.textKanjiHeader.setText(kanjiWords.headerText);
			this.textKanjiMeaning.setText(kanjiWords.meaningText);
		}

		this.updateButtonPrevNextState();

		super.onRestoreInstanceState(savedInstanceState);
	}

	@Override
	public void onBackPressed() {
		// Clean the cache:
		KanjiToWordActivity.loaderHandler.activity = null;
		KanjiToWordActivity.loaderThreadKanji.requestPrematureEnd();
		KanjiToWordActivity.loaderThreadWords.requestPrematureEnd();
		while (KanjiToWordActivity.loaderThreadWords.isAlive() || KanjiToWordActivity.loaderThreadKanji.isAlive()) {
			try {
				Thread.sleep(50);
			} catch (InterruptedException ie) {
			}
		}
		KanjiToWordActivity.loaderThreadWords = null;
		KanjiToWordActivity.loaderThreadKanji = null;
		KanjiToWordActivity.loaderHandler = null;

		if (null != KanjiToWordActivity.listKanjiWords) {
			KanjiToWordActivity.listKanjiWords.clear();
			KanjiToWordActivity.listKanjiWords = null;
		}

		if (null != this.progressDialog) {
			this.progressDialog.dismiss();
			this.progressDialog = null;
		}
		this.entryListFragment.clear();

		System.gc();
		System.runFinalization();

		super.onBackPressed();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.kanji_to_words, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		switch (item.getItemId()) {
		case R.id.kanjiToWordsMenuShowAll:
			this.showAllKanjis();
			break;
		case R.id.kanjiToWordsMenuLoadAll:
			if (null == this.progressDialog) {
				this.progressDialog = ProgressDialog.show(this, Common.EMPTY_STRING,
						Common.getString(R.string.loading), true);
			}
			KanjiToWordActivity.loaderThreadWords.loadAllWords(this.currentPosition);

			break;
		}
		return true;
	}

	public void setContentView() {
		Log.d(TAG, "setContentView " + R.layout.kanji_to_words);
		super.setContentView(R.layout.kanji_to_words);
		Log.d(TAG, "setContentView done");
		this.textKanjiHeader = (TextView) super.findViewById(R.id.kanjiToWordsTextHeader);
		this.textKanjiMeaning = (TextView) super.findViewById(R.id.kanjiToWordsTextMeaning);
		this.buttonNext = (ImageButton) super.findViewById(R.id.kanjiToWordsButtonNext);
		this.buttonPrevious = (ImageButton) super.findViewById(R.id.kanjiToWordsButtonPrevious);
		this.entryListFragment = (EntryListFragment) super.getSupportFragmentManager().findFragmentById(
				R.id.kanjiToWordsEntryList);

	}

	private void showAllKanjis() {
		final int numKanji = getNumKanjiWords();
		final int[] arrayCodepoint = new int[numKanji];
		for (int i = 0; i < numKanji; i++) {
			arrayCodepoint[i] = listKanjiWords.get(i).kanji.codepoint;
		}
		Intent intent = new Intent(this, KanjiListSelectActivity.class);
		intent.putExtra(KanjiListSelectActivity.TAG, arrayCodepoint);
		super.startActivityForResult(intent, KanjiToWordActivity.ALL_KANJI_CODE);
	}

	/**
	 * Result from the all kanjis activity
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (KanjiToWordActivity.ALL_KANJI_CODE == requestCode && resultCode >= 0) {
			this.currentPosition = resultCode;

			final int numKanji = getNumKanjiWords();
			int direction = 1;
			if (this.currentPosition >= numKanji - 1) {
				this.currentPosition = numKanji - 1;
				direction = -1;
			}

			KanjiToWordActivity.loaderThreadWords.setDisplayPosition(this.currentPosition, direction);
			this.updateDisplay();
			this.updateButtonPrevNextState();
		}
	}

	/**
	 * Click on the next kanji button
	 * 
	 * @param nextButton
	 */
	public void nextKanji(View nextButton) {

		final int numKanjis = getNumKanjiWords();
		if (this.currentPosition + 1 < numKanjis) {
			this.currentPosition++;
			KanjiToWordActivity.loaderThreadWords.setDisplayPosition(this.currentPosition, 1);
			this.updateDisplay();
			// Full scroll up.
			this.entryListFragment.getListView().scrollTo(0, 1);
		}

		this.updateButtonPrevNextState();
	}

	/**
	 * Click on the previous kanji button
	 * 
	 * @param previousButton
	 */
	public void previousKanji(View previousButton) {
		if (this.currentPosition > 0) {
			this.currentPosition--;
			KanjiToWordActivity.loaderThreadWords.setDisplayPosition(this.currentPosition, -1);
			this.updateDisplay();
			// Full scroll up.
			this.entryListFragment.getListView().scrollTo(0, 1);

		}

		this.updateButtonPrevNextState();
	}

	private void updateButtonPrevNextState() {
		final int numKanjis = getNumKanjiWords();

		if (0 == this.currentPosition) {
			this.buttonPrevious.setEnabled(false);
			this.buttonNext.setEnabled(true);
		} else if (this.currentPosition + 1 == numKanjis) {
			this.buttonPrevious.setEnabled(true);
			this.buttonNext.setEnabled(false);
		} else {
			this.buttonPrevious.setEnabled(true);
			this.buttonNext.setEnabled(true);
		}
	}

	private void updateDisplay() {

		final KanjiWords kanjiWords = KanjiToWordActivity.getKanjiWords(this.currentPosition);
		if (null != kanjiWords && null != kanjiWords.entries) {

			this.textKanjiHeader.setText(kanjiWords.headerText);
			this.textKanjiMeaning.setText(kanjiWords.meaningText);

			final ArrayList<Entry> entries = kanjiWords.entries;

			final EntryArrayAdapter listAdapter = (EntryArrayAdapter) this.entryListFragment.getListAdapter();
			listAdapter.setEntries(entries);

			if (this.defaultLimit != entries.size()) {
				final String msg = String.format(super.getString(R.string.toastNumWords), kanjiWords.headerText,
						listAdapter.getCount());
				Toast.makeText(this, msg, Toast.LENGTH_LONG).show();
			}
		} else {
			synchronized (KanjiToWordActivity.loaderThreadWords) {
				if (null == this.progressDialog) {
					this.progressDialog = ProgressDialog.show(this, Common.EMPTY_STRING,
							Common.getString(R.string.loading), true);
				}
				KanjiToWordActivity.loaderThreadWords.setNotifyLoaded();
			}
		}

	}

	private static void addKanjiWords(KanjiWords kanjiWords) {
		synchronized (TAG) {
			KanjiToWordActivity.listKanjiWords.add(kanjiWords);
		}
	}

	private static KanjiWords getKanjiWords(int pos) {
		KanjiWords kanjiWords = null;
		synchronized (TAG) {
			if (pos >= 0 && pos < KanjiToWordActivity.listKanjiWords.size()) {
				kanjiWords = KanjiToWordActivity.listKanjiWords.get(pos);
			}
		}
		return kanjiWords;
	}

	private static KanjiWords getNextKanjiWordsToLoad(int startPos, int direction) {
		KanjiWords kw = null;
		synchronized (TAG) {
			final int numKW = KanjiToWordActivity.listKanjiWords.size();

			int pos = startPos;
			boolean loop = (pos >= 0 && pos < numKW);
			while (loop) {
				kw = KanjiToWordActivity.listKanjiWords.get(pos);
				pos += direction;
				if (null == kw.entries) {
					loop = false;
				} else if (pos < 0 || pos >= numKW) {
					loop = false;
					kw = null;
				}
			}
		}

		return kw;
	}

	private static int getNumKanjiWords() {
		synchronized (TAG) {
			return KanjiToWordActivity.listKanjiWords.size();
		}
	}

	/**
	 * Put in cache the results.
	 * 
	 */
	private static class KanjiWords {
		final Kanji kanji;
		ArrayList<Entry> entries;
		String meaningText;
		String headerText;
		final int position;

		KanjiWords(Kanji kanji, int pos) {
			this.kanji = kanji;
			this.entries = null;
			this.meaningText = null;
			this.position = pos;
		}

		void setNumWords(int num) {
			synchronized (TAG) {
				this.entries = new ArrayList<Entry>(num);
			}
		}

		void addWord(Entry entry) {
			synchronized (TAG) {
				this.entries.add(entry);
			}
		}
	}

	/**
	 * The thread who load the kanjis.
	 */
	private static class LoaderThreadKanji extends Thread {

		private final String selection;
		private final String[] selectionArgs;
		private boolean prematureEnd;
		final ArrayList<Language> languages;

		LoaderThreadKanji(String selection, String[] selectionArgs) {
			this.selection = selection;
			this.selectionArgs = selectionArgs;
			this.prematureEnd = false;
			this.languages = Preferences.getLangsPriorities(DatabaseType.kanji);
			super.setName("KanjiToWords: LoaderThreadKanji");
		}

		@SuppressLint("FieldGetter")
		@Override
		public void run() {

			final KanjiDB.KanjiDBIterator kanjiIterator = KanjiDB.getKanjiIterator(this.selection, this.selectionArgs,
					true, false, true);

			Kanji kanji = kanjiIterator.first();
			int count = 0;
			while (kanji != null) {
				this.loadKanji(kanji, count);

				if (count == LoaderThreadWords.MAX_ADVANCE) {
					// We have load enough kanji to start the loading of words:
					KanjiToWordActivity.loaderThreadWords.start();
				}

				if (this.isEndRequested()) {
					Log.i(TAG, "LoaderThreadKanji premature end");
					kanji = null;
				} else {
					kanji = kanjiIterator.next();
				}

				count++;
			} // while

			kanjiIterator.close();
		}

		synchronized boolean isEndRequested() {
			return this.prematureEnd;
		}

		synchronized void requestPrematureEnd() {
			this.prematureEnd = true;
		}

		/**
		 * Init the KanjiWords
		 * 
		 * @param kanji
		 * @param langs
		 * @param langsCount
		 */
		private void loadKanji(Kanji kanji, int pos) {
			final KanjiWords kanjiWords = new KanjiWords(kanji, pos);

			final String kanjiString = new String(Character.toChars(kanji.codepoint));
			kanjiWords.headerText = kanjiString;

			String meaning = null;
			if (null != kanji.meanings) {
				// TODO: duplicate code in LearnWritingActivity
				final ArrayList<Language> lLanguages = this.languages;
				final int langsCount = lLanguages.size();
				for (int i = 0; i < langsCount; i++) {
					Language language = lLanguages.get(i);
					meaning = kanji.meanings.get(language);
					if (null != meaning) {
						break;
					}
				}
			}
			if (null == meaning) {
				meaning = Common.EMPTY_STRING;
			}
			kanjiWords.meaningText = meaning.replace('/', '\n');

			KanjiToWordActivity.addKanjiWords(kanjiWords);
		}

	}

	/**
	 * Thread which load words for a specific kanji.
	 * 
	 */
	private static class LoaderThreadWords extends Thread {
		private static final int MAX_ADVANCE = 5;
		private DictionaryDB.KanjiToWordsIterator wordsIterator;
		private int userPosition;
		/**
		 * <ul>
		 * <li>1 for advance forward (next kanji)</li>
		 * <li>-1 for advance forward (previous kanji)</li>
		 * </ul>
		 */
		private int userDirection;
		private int currentPosition;
		private boolean notifyLoaded;
		private boolean requestEnd;
		private int loadAllWordsPosition;

		public LoaderThreadWords() {
			this.userPosition = 0;
			this.userDirection = 1;
			this.currentPosition = 0;
			this.loadAllWordsPosition = -1;
			this.notifyLoaded = true;
			this.requestEnd = false;
			super.setName("KanjiToWords: LoaderThreadWords");
		}

		@Override
		public void run() {

			this.wordsIterator = new DictionaryDB.KanjiToWordsIterator();

			boolean loop = true;
			while (loop) {
				KanjiWords kanjiWords = null;

				this.doLoadAllWords();

				kanjiWords = KanjiToWordActivity.getNextKanjiWordsToLoad(this.currentPosition, this.userDirection);
				if (null != kanjiWords) {

					this.currentPosition = kanjiWords.position;
					// Fetch the words from the database
					this.loadWords(kanjiWords, false);
					// Tell the UI that we have load the words it waited
					this.sendNotification();

				} else if (this.userDirection == 1) {
					this.currentPosition = KanjiToWordActivity.getNumKanjiWords() - 1;
				} else {
					this.currentPosition = 0;
				}

				loop = !this.waitIfNeeded();
			}

			this.wordsIterator.close();
			this.wordsIterator = null;
		}

		private synchronized boolean waitIfNeeded() {
			// need waiting?
			final int cP = this.currentPosition;
			final int uP = this.userPosition;
			final int uD = this.userDirection;

			// yes if we have enough advance
			boolean nWait = Math.abs(cP - uP) > MAX_ADVANCE;
			// yes if we are at the beginning
			nWait = nWait || (cP == 0 && uD == -1);
			// yes if we are at the end
			nWait = nWait || ((cP + 1) == KanjiToWordActivity.getNumKanjiWords() && uD == 1);
			// no if we need to stop
			if (nWait && !this.requestEnd) {
				try {
					KanjiToWordActivity.loaderHandler.sendLoadWait();
					super.wait();
				} catch (InterruptedException ie) {
				}
			}

			return this.requestEnd;
		}

		private synchronized void sendNotification() {
			if (this.notifyLoaded) {
				KanjiToWordActivity.loaderHandler.sendLoadDone(this.currentPosition);
				this.notifyLoaded = false;
			}
		}

		synchronized void setNotifyLoaded() {
			this.notifyLoaded = true;
		}

		synchronized void loadAllWords(int pos) {
			this.loadAllWordsPosition = pos;
			super.notify();
		}

		synchronized void doLoadAllWords() {
			if (this.loadAllWordsPosition != -1) {
				KanjiWords kanjiWords = KanjiToWordActivity.getKanjiWords(this.loadAllWordsPosition);
				this.loadWords(kanjiWords, true);
				KanjiToWordActivity.loaderHandler.sendLoadDone(this.loadAllWordsPosition);
				this.loadAllWordsPosition = -1;

				System.runFinalization();
				System.gc();
			}
		}

		synchronized void setDisplayPosition(int newPos, int direction) {
			this.userPosition = newPos;
			this.currentPosition = newPos;
			this.userDirection = direction;
			super.notify();
		}

		synchronized void requestPrematureEnd() {
			this.requestEnd = true;
			super.notify();
		}

		/**
		 * Load the words from the dictionary
		 * 
		 * @param kanjiWords
		 * @param langs
		 */
		private void loadWords(KanjiWords kanjiWords, boolean noLimit) {

			final DictionaryDB.KanjiToWordsIterator wi = this.wordsIterator;

			if (wi.setCodepoint(kanjiWords.kanji.codepoint, noLimit)) {
				final int wiCount = wi.getCount();
				kanjiWords.setNumWords(wiCount);
				// Fetch entries for the dictionary
				final ArrayList<Entry> entries = new ArrayList<Entry>(wiCount);
				do {
					final Entry entry = wi.next();
					if (null != entry)
						entries.add(entry);
				} while (wi.hasNext());

				// Sort elements
				Collections.sort(entries);

				// Transform them to an instance of Word.
				// final int numLangs = langsDict.size();
				final int numEntrie = entries.size();
				for (int i = 0; i < numEntrie; i++) {
					kanjiWords.addWord(entries.get(i));
				}
			} else {
				kanjiWords.setNumWords(0);
				// Log.i(TAG, "Can't get words for kanji " +
				// kanjiWords.kanji.codepoint);
			}
		}
	}

	/**
	 * Make the link between the UI Thread and the loader thread.
	 */
	private static class LoaderHandler extends Handler {
		private KanjiToWordActivity activity;

		public LoaderHandler(KanjiToWordActivity activity) {
			super();
			this.activity = activity;
		}

		void sendLoadDone(int position) {
			Message message = Message.obtain(this);
			Bundle bundle = message.getData();
			bundle.clear();
			if (position >= 0) {
				bundle.putInt(LOAD_KANJIS_SINGLE_DONE, position);
			} else {
				bundle.putInt(LOAD_KANJIS_DONE, position);
			}
			KanjiToWordActivity.loaderHandler.sendMessage(message);
		}

		void sendLoadWait() {
			Message message = Message.obtain(this);
			Bundle bundle = message.getData();
			bundle.clear();
			bundle.putBoolean(LOAD_KANJIS_WAIT, true);
			KanjiToWordActivity.loaderHandler.sendMessage(message);
		}

		@Override
		public void handleMessage(Message msg) {
			if (null != this.activity) {
				Bundle bundle = msg.getData();
				if (bundle.containsKey(LOAD_KANJIS_SINGLE_DONE)) {

					this.activity.updateDisplay();

					if (this.activity.progressDialog != null) {
						this.activity.progressDialog.dismiss();
						this.activity.progressDialog = null;
					}

				} else if (bundle.containsKey(LOAD_KANJIS_WAIT)) {
					// In some case there is the progress dialog
					// show indefinitely.
					if (this.activity.progressDialog != null) {
						this.activity.progressDialog.dismiss();
						this.activity.progressDialog = null;
					}
				} else if (bundle.containsKey(LOAD_KANJIS_DONE)) {

				}
			}
		}
	}

}
