/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.models.history;

import java.util.ArrayList;
import java.util.Date;

import org.vono.narau.utils.Language;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * Result on an offline search.
 * 
 */
public class OfflineResult extends SearchResult {
	public final ArrayList<Integer> dictionaryIds;

	public OfflineResult(String search, Language srcLang, int numResults) {
		this(new Date(), search, srcLang, numResults);
	}

	public OfflineResult(Date date, String search, Language srcLang, int numResults) {
		super(date, search, numResults, srcLang, null);
		this.dictionaryIds = new ArrayList<Integer>(numResults);
	}

	protected OfflineResult(Parcel in) {
		super(in);
		final int count = in.readInt();
		this.dictionaryIds = new ArrayList<Integer>(count);
		final ArrayList<Integer> ids = this.dictionaryIds;
		for (int i = 0; i < count; i++) {
			ids.add(in.readInt());
		}
	}

	@Override
	public String getDisplayLanguage() {
		return this.sourceLang.displayName;
	}

	@Override
	public long getId() {
		return ((long) this.searchTerm.hashCode()) + ((long) this.sourceLang.langISO3.hashCode());
	}

	@Override
	public void writeToParcel(Parcel out, int flags) {
		super.writeToParcel(out, flags);
		final ArrayList<Integer> ids = this.dictionaryIds;
		final int count = ids.size();
		out.writeInt(count);
		for (int i = 0; i < count; i++) {
			out.writeInt(ids.get(i));
		}
	}

	public static final Parcelable.Creator<OfflineResult> CREATOR = new Parcelable.Creator<OfflineResult>() {
		public OfflineResult createFromParcel(Parcel in) {
			return new OfflineResult(in);
		}

		public OfflineResult[] newArray(int size) {
			return new OfflineResult[size];
		}
	};

}
