/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau;

import java.io.IOException;
import java.io.InputStream;

import android.app.Activity;
import android.os.Bundle;
import android.text.Html;
import android.text.Spanned;
import android.text.method.LinkMovementMethod;
import android.util.Log;
import android.widget.TabHost;
import android.widget.TabHost.OnTabChangeListener;
import android.widget.TabHost.TabSpec;
import android.widget.TextView;

/**
 * Display the About text, GPLv3 license and the thanks texts.
 * 
 */
public class AboutActivity extends Activity implements OnTabChangeListener {
	private static final String TAG = AboutActivity.class.getSimpleName();

	private static final String TAB_ABOUT = "about";
	private static final String TAB_LICENSE = "license";
	private static final String TAB_THANKS = "thanks";

	private static final String SMALL_ABOUT = "<h1 style=\"text-align: center;\">Narau 習う (ならう)</h1><br/>"
			+ "<i>To take lessons in/to be taught/to learn (from a teacher)/to study (under a teacher)/to get training in.</i><br/>"
			+ "<br/>"
			+ "&bull; An offline dictionary (Japanese / English,French,German,Dutch,Russian, Spannish), based on JMDict (Japanese-Multilingual Dictionary)<br/>"
			+ "&bull; An online dictionary (MyMemory / Microsoft Translator / Google Translate)<br/>"
			+ "&bull; Hiragana/Katakana charts<br/>"
			+ "&bull; A Kana/Kanji drawing panel<br/>"
			+ "&bull; Kana/Kanji drill (English, French, Portuguese, Spanish). The kanjis informations comes from Kanjivg and KANJIDIC2 databases.<br/>"
			+ "<br/>" + "Web site: <a href=\"https://launchpad.net/narau\">https://launchpad.net/narau</a><br/>";

	private static final String SMALL_GPL = "Narau  Copyright (C) 2012 Yvon TANGUY<br/>\n"
			+ "<br/>\n"
			+ "This program is free software: you can redistribute it and/or modify "
			+ "it under the terms of the GNU General Public License as published by "
			+ "the Free Software Foundation, either version 3 of the License, or "
			+ "(at your option) any later version.<br/>\n"
			+ "<br/>\n"
			+ "This program is distributed in the hope that it will be useful, "
			+ "but WITHOUT ANY WARRANTY; without even the implied warranty of "
			+ "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the "
			+ "GNU General Public License for more details.<br/>\n"
			+ "<br/>\n"
			+ "You should have received a copy of the GNU General Public License "
			+ "along with this program.  If not, see <a href=\"http://www.gnu.org/licenses\">http://www.gnu.org/licenses</a>.";

	private static final String SMALL_THANKS = "Thanks the world!";

	private Spanned htmlAbout;
	private Spanned htmlLicense;
	private Spanned htmlThanks;

	private TextView textViewAbout;
	private TextView textViewLicense;
	private TextView textViewThanks;

	public AboutActivity() {
		this.htmlAbout = null;
		this.htmlLicense = null;
		this.htmlThanks = null;
		this.textViewAbout = null;
		this.textViewLicense = null;
		this.textViewThanks = null;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		super.setContentView(R.layout.about);

		this.textViewAbout = (TextView) super.findViewById(R.id.aboutTextViewAbout);
		this.textViewAbout.setMovementMethod(LinkMovementMethod.getInstance());

		this.textViewLicense = (TextView) super.findViewById(R.id.aboutTextViewLicense);
		this.textViewLicense.setMovementMethod(LinkMovementMethod.getInstance());

		this.textViewThanks = (TextView) super.findViewById(R.id.aboutTextViewThanks);
		this.textViewThanks.setMovementMethod(LinkMovementMethod.getInstance());

		TabHost mTabHost = (TabHost) super.findViewById(android.R.id.tabhost);
		mTabHost.setup();
		mTabHost.setOnTabChangedListener(this);

		TabSpec mTabSpec = mTabHost.newTabSpec(TAB_ABOUT);
		mTabSpec.setIndicator(this.getString(R.string.aboutAbout),
				super.getResources().getDrawable(R.drawable.ic_tab_narau));
		mTabSpec.setContent(R.id.aboutScollViewAbout);
		mTabHost.addTab(mTabSpec);

		mTabSpec = mTabHost.newTabSpec(TAB_LICENSE);
		mTabSpec.setIndicator(this.getString(R.string.aboutLicense),
				super.getResources().getDrawable(R.drawable.ic_tab_gplv3));
		mTabSpec.setContent(R.id.aboutScollViewLicense);
		mTabHost.addTab(mTabSpec);

		mTabSpec = mTabHost.newTabSpec(TAB_THANKS);
		mTabSpec.setIndicator(this.getString(R.string.aboutThanks),
				super.getResources().getDrawable(R.drawable.ic_tab_thanks));
		mTabSpec.setContent(R.id.aboutScollViewThanks);
		mTabHost.addTab(mTabSpec);

	}

	public void onTabChanged(String tabId) {
		if (TAB_ABOUT.equals(tabId)) {
			if (null == this.htmlAbout) {
				StringBuilder sb = this.getHtml(R.raw.about, SMALL_ABOUT);
				sb.append(this.getHtml(R.raw.changelog, ""));
				sb.append("</body></html>");
				this.htmlAbout = Html.fromHtml(sb.toString());
			}
			this.textViewAbout.setText(this.htmlAbout);
		} else if (TAB_LICENSE.equals(tabId)) {
			if (null == this.htmlLicense) {
				this.htmlLicense = Html.fromHtml(this.getHtml(R.raw.gpl, SMALL_GPL).toString());
			}
			this.textViewLicense.setText(this.htmlLicense);
		} else if (TAB_THANKS.equals(tabId)) {
			if (null == this.htmlThanks) {
				this.htmlThanks = Html.fromHtml(this.getHtml(R.raw.thanks, SMALL_THANKS).toString());
			}
			this.textViewThanks.setText(this.htmlThanks);
		}
	}

	/**
	 * Read the file "gpl.html" in the assets directory.
	 * 
	 * @return The HTML file.
	 */
	private StringBuilder getHtml(int resId, String defaultText) {
		final InputStream inputStream = super.getResources().openRawResource(resId);
		final byte[] buffer = new byte[512];
		final StringBuilder sbHTML = new StringBuilder(512);

		try {
			boolean eof = false;
			do {
				final int count = inputStream.read(buffer);
				if (-1 == count) {
					eof = true;
				} else {
					final String str = new String(buffer, 0, count, "UTF-8");
					sbHTML.append(str);
				}
			} while (!eof);
		} catch (IOException ioe) {
			Log.e(TAG, ioe.getMessage(), ioe);
			sbHTML.delete(0, sbHTML.length());
			sbHTML.append(defaultText);
		}

		try {
			inputStream.close();
		} catch (IOException ioe) {
			Log.e(TAG, ioe.getMessage(), ioe);
			sbHTML.delete(0, sbHTML.length());
			sbHTML.append(defaultText);
		}

		return sbHTML;
	}

}
