/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.vono.narau.common.AppException;
import org.vono.narau.common.Common;
import org.vono.narau.db.CommonDB;
import org.vono.narau.db.DatabaseType;
import org.vono.narau.download.DownloaderActivity;
import org.vono.narau.preferences.Preferences;
import org.vono.narau.preferences.PreferencesActivity;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.compat.view.MenuCompat;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.NotificationCompat;
import android.support.v4.app.TaskStackBuilder;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

/**
 * 
 * <p>
 * <b>Narau</b> 習う (ならう)<br/>
 * to take lessons in/to be taught/to learn (from a teacher)/to study (under a teacher)/to get
 * training in
 * </p>
 * The menu of this application.
 */
public class Narau extends FragmentActivity implements OnClickListener {

	/**
	 * Log tag
	 */
	private static final String TAG = Narau.class.getSimpleName();
	private static Narau instance = null;

	/** Called when the activity is created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		Narau.instance = this;

		if (DownloaderActivity.isActive()) {
			this.showDownload(DownloaderActivity.LauncherSource.FIRST_RUN);
		} else {
			this.doInit();
		}
	}

	private void doInit() {
		final Context context = this.getApplicationContext();
		try {
			Common.init(context);

			if (!Preferences.isFirstRun()) {
				this.showMenuView();
			} else {
				this.showDownload(DownloaderActivity.LauncherSource.FIRST_RUN);
			}

		} catch (AppException ae) {
			// show the init error view
			super.setContentView(R.layout.narau_init_error);
			((Button) super.findViewById(R.id.initErrorButtonQuit)).setOnClickListener(this);
			TextView tv = (TextView) super.findViewById(R.id.initErrorTextView);
			tv.setText(ae.getMessage());
		}
	}

	@Override
	public void onBackPressed() {
		Common.clean();
		super.onBackPressed();
	}

	@Override
	protected void onStop() {
		MenuCompat.closeMenu(R.menu.main);
		super.onStop();
	}

	/**
	 * 
	 * @param v
	 *            the menu_init_error.xml quit button (initErrorButtonQuit).
	 */
	public void onClickButtonCancel(View v) {
		QuitThread.startQuitThread();
		super.finish();
	}

	/**
	 * Show the menu view
	 */
	public void showMenuView() {

		final ArrayList<MenuActivity> listActivities = new ArrayList<MenuActivity>(5);

		final Intent intent = new Intent(Intent.ACTION_MAIN, null);
		intent.addCategory("org.vono.narau.MENU");

		final PackageManager pm = getPackageManager();
		final List<ResolveInfo> infos = pm.queryIntentActivities(intent, 0);
		final int count = infos.size();
		for (int i = 0; i < count; i++) {
			final ResolveInfo info = infos.get(i);

			final CharSequence labelSeq = info.loadLabel(pm);
			final String label = labelSeq != null ? labelSeq.toString() : info.activityInfo.name;

			// int v = info.activityInfo.applicationInfo.descriptionRes;
			String description = null;
			if (0 != info.activityInfo.applicationInfo.descriptionRes) {
				description = super.getString(info.activityInfo.applicationInfo.descriptionRes);
			}

			final Intent target = new Intent();
			target.setClassName(info.activityInfo.applicationInfo.packageName, info.activityInfo.name);

			// Check if there is the database for this item
			boolean enabled = true;
			try {
				final Class<?> aClass = super.getClassLoader().loadClass(info.activityInfo.name);
				final Method m = aClass.getMethod("haveDatabase", new Class<?>[0]);
				enabled = (Boolean) m.invoke(null, new Object[0]);

			} catch (ClassNotFoundException nfee) {
				Log.e(TAG, nfee.getMessage(), nfee);
			} catch (SecurityException se) {
				Log.e(TAG, se.getMessage(), se);
			} catch (NoSuchMethodException nsme) {
				Log.e(TAG, info.activityInfo.name + ": " + nsme.getMessage(), nsme);
			} catch (IllegalArgumentException iae) {
				Log.e(TAG, iae.getMessage(), iae);
			} catch (IllegalAccessException iae) {
				Log.e(TAG, iae.getMessage(), iae);
			} catch (InvocationTargetException ite) {
				Log.e(TAG, ite.getMessage(), ite);
			}

			Drawable dIcon = null;
			if (0 != info.activityInfo.icon) {
				dIcon = super.getResources().getDrawable(info.activityInfo.icon);
			}

			MenuActivity ma = new MenuActivity(label, description, target, dIcon);
			ma.enabled = enabled;
			listActivities.add(ma);
		}

		MainMenuFragment.setList(listActivities, super.getLayoutInflater());
		super.setContentView(R.layout.narau);

		this.showMissingDatabaseNotification();
	}

	private void showMissingDatabaseNotification() {
		final boolean haveDict = CommonDB.haveDB(DatabaseType.dictonary);
		final boolean haveKanji = CommonDB.haveDB(DatabaseType.kanji);
		if (!haveDict || !haveKanji) {

			final String message;
			if (!haveDict && !haveKanji) {
				message = super.getString(R.string.notificationDBMissing);
			} else {
				final String template = super.getString(R.string.notificationOneBDMissing);
				if (!haveDict) {
					message = String.format(template, super.getString(R.string.dbDictionary));
				} else {
					message = String.format(template, super.getString(R.string.dbKanjiKana));
				}
			}

			final Intent intent = new Intent(this, DownloaderActivity.class);
			intent.putExtra(DownloaderActivity.UPDATE_PROCESS, DownloaderActivity.LauncherSource.NOTIFICATION);

			final TaskStackBuilder taskStackBuilder = TaskStackBuilder.create(this);
			taskStackBuilder.addParentStack(DownloaderActivity.class);
			taskStackBuilder.addNextIntent(intent);

			final PendingIntent pendingIntent = taskStackBuilder.getPendingIntent(0, 0);
			final String title = super.getString(R.string.app_name);
			final NotificationCompat.Builder builder = new NotificationCompat.Builder(this);
			builder.setTicker(message);
			builder.setSmallIcon(R.drawable.ic_stat_notify_db_update);
			builder.setContentInfo(message);
			builder.setContentText(message);
			builder.setContentTitle(title);
			builder.setAutoCancel(true);
			builder.setContentIntent(pendingIntent);

			NotificationManager nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
			Notification notification = builder.build();
			nm.notify(R.id.download_db_notification, notification);
		}
	}

	public static void finishOnUpdateDB() {
		if (null != Narau.instance) {
			Narau.instance.finish();
			Narau.instance = null;
		}
	}

	private void showDownload(DownloaderActivity.LauncherSource source) {
		Intent intent = new Intent(super.getApplicationContext(), DownloaderActivity.class);
		intent.putExtra(DownloaderActivity.UPDATE_PROCESS, source);
		super.startActivity(intent);
		super.finish();
	}

	@Override
	public boolean onKeyUp(int keyCode, KeyEvent event) {
		Menu menu = MenuCompat.onKeyUp(this, keyCode, R.menu.main);
		if (null != menu) {
			return true;
		}
		return super.onKeyUp(keyCode, event);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		boolean downloading = false;
		if (!downloading) {

			switch (item.getItemId()) {
			case R.id.menuMainQuit:
				this.onClickButtonCancel(null);
				break;
			case R.id.menuMainAbout:
				Intent mIntent = new Intent(this, AboutActivity.class);
				super.startActivity(mIntent);
				break;
			case R.id.menuMainUpdateDB:
				this.showDownload(DownloaderActivity.LauncherSource.UPDATE);
				break;
			case R.id.menuMainPreferences:
				mIntent = new Intent(this.getApplicationContext(), PreferencesActivity.class);
				super.startActivity(mIntent);
				break;
			default:
				return super.onOptionsItemSelected(item);
			}
		}
		return true;
	}

	/**
	 * An activity displayed in the menu.
	 */
	static class MenuActivity {
		final String name;
		final String description;
		final Intent intent;
		final Drawable icon;
		boolean enabled;

		MenuActivity(String name, String description, Intent intent, Drawable drawable) {
			this.name = name;
			this.description = description;
			this.intent = intent;
			this.icon = drawable;
			if (null != drawable) {
				this.icon.setBounds(0, 0, this.icon.getIntrinsicWidth(), this.icon.getIntrinsicHeight());
			}
			this.enabled = true;
		}
	}

	/**
	 * The list adapter.
	 */
	static class MenuAdapter extends BaseAdapter {
		private final List<MenuActivity> listActivities;
		private final LayoutInflater layoutInflater;

		public MenuAdapter(List<MenuActivity> items, LayoutInflater layoutInflater) {
			this.listActivities = items;
			this.layoutInflater = layoutInflater;
		}

		public int getCount() {
			return this.listActivities.size();
		}

		public Object getItem(int position) {
			return this.listActivities.get(position);
		}

		public long getItemId(int position) {
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			View cView = convertView;
			if (null == cView) {
				cView = this.layoutInflater.inflate(R.layout.narau_item, parent, false);
			}

			final TextView tvTitle = (TextView) cView.findViewById(android.R.id.title);
			final TextView tvSummary = (TextView) cView.findViewById(android.R.id.summary);
			final ImageView ivIcon = (ImageView) cView.findViewById(android.R.id.icon);

			final MenuActivity item = this.listActivities.get(position);
			tvTitle.setText(item.name);
			ivIcon.setImageDrawable(item.icon);
			tvSummary.setText(item.description);
			if (null == item.description) {
				tvSummary.setVisibility(View.GONE);
			} else {
				tvSummary.setVisibility(View.VISIBLE);
			}

			cView.setEnabled(item.enabled);
			tvTitle.setEnabled(item.enabled);
			tvSummary.setEnabled(item.enabled);
			ivIcon.setEnabled(item.enabled);

			return cView;
		}

	}

	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.initErrorButtonQuit:
			this.onClickButtonCancel(v);
			break;
		}

	}

}