/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.learn.reading;

import org.vono.narau.common.AppException;
import org.vono.narau.common.Common;
import org.vono.narau.common.Common.KanaType;
import org.vono.narau.common.kanji.KanjiArrayAdapter;
import org.vono.narau.db.CommonDB;
import org.vono.narau.db.DatabaseType;
import org.vono.narau.models.kanji.KanjiData;

import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ListView;

/**
 * 
 *
 */
public class LearnReadingMenuActivity extends ListActivity implements OnItemClickListener {

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		if (!Common.isInitDone()) {
			// Avoid a double crash, in for example KanjiToWordActivity crash
			// LearnReadingMenuActivity will be launch in a new process
			// but the common init is not done...
			try {
				Common.init(super.getApplicationContext());
			} catch (AppException ae) {
				super.finish();
				return;
			}
		}

		ListView listView = super.getListView();

		final boolean dbAvail = CommonDB.haveDB(DatabaseType.dictonary.getValue() | DatabaseType.kanji.getValue());
		int disabledTypeMask = 0;
		if (!dbAvail) {
			disabledTypeMask = KanjiData.KANJI;
		}
		KanjiArrayAdapter kaa = new KanjiArrayAdapter(this.getApplicationContext(), KanaChartActivity.TYPE_MASK
				| KanjiToWordActivity.TYPE_MASK, disabledTypeMask);

		listView.setAdapter(kaa);
		listView.setOnItemClickListener(this);
	}

	public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
		if (view.isEnabled()) {

			Object tag = view.getTag();

			if (tag instanceof KanjiData.WritingCategory) {

				KanjiData.WritingCategory wc = (KanjiData.WritingCategory) tag;
				Intent intent = null;
				if (wc.type == KanaType.KANJI) {
					intent = new Intent(this.getApplicationContext(), KanjiToWordActivity.class);
				} else {
					intent = new Intent(this.getApplicationContext(), KanaChartActivity.class);
				}
				intent.putExtra(KanjiArrayAdapter.ITEM_CATEGORY, wc);
				super.startActivity(intent);
			}
		}
	}

	public static boolean haveDatabase() {
		return true;
	}

}
