/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.models.history;

import java.util.Date;

import org.vono.narau.utils.Language;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * One previous dictionary search.
 * 
 */
public abstract class SearchResult implements Comparable<SearchResult>, Parcelable {
	private static final String NULL = "null";
	public final Date date;
	public final String searchTerm;
	public final Language sourceLang;
	/**
	 * <b>/!\ May be null.</b>
	 */
	public final Language destLang;
	public int numResults;

	public SearchResult(Date date, String searchTerm, int numResults, Language srcLang, Language dstLang) {
		this.date = date;
		this.searchTerm = searchTerm;
		this.numResults = numResults;
		this.sourceLang = srcLang;
		this.destLang = dstLang;
	}

	protected SearchResult(Parcel in) {
		this.date = new Date(in.readLong());
		this.searchTerm = in.readString();
		this.sourceLang = Language.MAP_ISO3.get(in.readString());
		final String destLangIso3 = in.readString();
		if (NULL.equals(destLangIso3)) {
			this.destLang = null;
		} else {
			this.destLang = Language.MAP_ISO3.get(destLangIso3);
		}
		this.numResults = in.readInt();

	}

	public int compareTo(SearchResult another) {
		// Most recent first.
		return another.date.compareTo(this.date);
	}

	@Override
	public boolean equals(Object o) {
		boolean equal = false;
		if (o == this) {
			equal = true;
		} else if (o instanceof SearchResult) {
			SearchResult sr = (SearchResult) o;
			equal = (this.getId() == sr.getId());
		}
		return equal;
	}

	@Override
	public int hashCode() {
		return Long.valueOf(this.getId()).hashCode();
	}

	public abstract String getDisplayLanguage();

	public abstract long getId();

	public int describeContents() {
		// Parcelable
		return 0;
	}

	public void writeToParcel(Parcel out, int flags) {
		out.writeLong(this.date.getTime());
		out.writeString(this.searchTerm);
		out.writeString(this.sourceLang.langISO3);
		if (null == this.destLang) {
			out.writeString(NULL);
		} else {
			out.writeString(this.destLang.langISO3);
		}
		out.writeInt(this.numResults);
	}

}
