/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.preferences;

import java.util.ArrayList;

import org.vono.narau.R;
import org.vono.narau.db.CommonDB;
import org.vono.narau.db.DatabaseType;
import org.vono.narau.utils.Language;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.preference.DialogPreference;
import android.util.AttributeSet;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ListView;

/**
 * The preferences to be able to change the lang search order.
 * 
 */
public class LangListPreference extends DialogPreference implements OnClickListener {
	// private static final String TAG =
	// LangListPreference.class.getSimpleName();

	/** The dialog, if it is showing. */
	private Dialog mDialog;
	private LangArrayAdapter langArrayAdapter;
	private DatabaseType origin;

	public LangListPreference(Context context, AttributeSet attrs) {
		super(context, attrs);

		this.origin = DatabaseType.valueOf(attrs.getAttributeValue(null, "db_origin"));
		this.langArrayAdapter = null;
		this.mDialog = null;
	}

	@Override
	protected void showDialog(Bundle state) {
		boolean initDone = true;
		if (null == this.langArrayAdapter) {
			initDone = this.init();
		}
		if (initDone) {
			Context context = this.getContext();
			final Dialog dialog = new Dialog(context, 0);
			dialog.setContentView(R.layout.preference_dialog_dictionary_langs);
			dialog.getWindow().setBackgroundDrawableResource(R.color.transparent_black);

			if (this.origin == DatabaseType.dictonary) {
				dialog.setTitle(R.string.dictionary);
			} else {
				dialog.setTitle(R.string.kanji);
			}

			dialog.setCancelable(true);
			Button button = (Button) dialog.findViewById(R.id.prefDialogLangOrderButtonOk);
			button.setOnClickListener(this);
			button = (Button) dialog.findViewById(R.id.prefDialogLangOrderButtonCancel);
			button.setOnClickListener(this);

			ListView listView = (ListView) dialog.findViewById(R.id.prefDialogLangOrderList);

			listView.setAdapter(this.langArrayAdapter);

			this.mDialog = dialog;

			dialog.show();
		}
	}

	public void onClick(View v) {
		this.mDialog.dismiss();

		if (v.getId() == R.id.prefDialogLangOrderButtonOk) {
			Preferences.setLangsPriorities(this.origin, this.langArrayAdapter.getList());
		}
	}

	private boolean init() {

		if (CommonDB.haveDB(this.origin)) {
			final ArrayList<Language> languages = Preferences.getLangsPriorities(this.origin);
			LangArrayAdapter arrayAdapter = new LangArrayAdapter(this.getContext(), languages);
			this.langArrayAdapter = arrayAdapter;

			return true;
		}
		return false;
	}

}
