/*
 *  Copyright (C) 2012 Yvon TANGUY
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.vono.narau.preferences;

import org.vono.narau.R;
import org.vono.narau.common.kanji.KanjiViewDialogSeekBar;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.preference.DialogPreference;
import android.util.AttributeSet;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;

/**
 * A preference dialog that show a seekbar
 * 
 * @see KanjiViewDialogSeekBar
 * 
 */
public class SeekBarPreference extends DialogPreference implements OnClickListener,
		OnSeekBarChangeListener {
	private Dialog mDialog;
	private final float fDivider;
	private final float fMin;
	private final float fMax;
	private final float fDefault;
	private float fValue;
	private TextView textView;

	public SeekBarPreference(Context context, AttributeSet attrs) {
		super(context, attrs);

		int id = attrs.getAttributeResourceValue(null, "min_value", -1);
		String min = context.getResources().getString(id);

		id = attrs.getAttributeResourceValue(null, "max_value", -1);
		String max = context.getResources().getString(id);

		id = attrs.getAttributeResourceValue(null, "divider", -1);
		String divider = context.getResources().getString(id);

		this.fMin = Float.parseFloat(min);
		this.fMax = Float.parseFloat(max);
		this.fDivider = Float.parseFloat(divider);
		this.fDefault = Preferences.getFloat(super.getKey());
		this.fValue = this.fDefault;
	}

	@Override
	protected void showDialog(Bundle state) {
		Context context = this.getContext();
		final Dialog dialog = new Dialog(context, 0);
		dialog.setContentView(R.layout.preference_dialog_seekbar);
		dialog.setTitle(super.getTitle());

		dialog.setCancelable(true);
		Button button = (Button) dialog.findViewById(R.id.prefDialogSeekBarButtonOk);
		button.setOnClickListener(this);

		SeekBar seekBar = (SeekBar) dialog.findViewById(R.id.prefDialogSeekBarBar);
		seekBar.setMax((int) (this.fDivider * (this.fMax - this.fMin)));
		seekBar.setProgress((int) (this.fDivider * (this.fDefault - this.fMin)));

		seekBar.setOnSeekBarChangeListener(this);

		this.textView = (TextView) dialog.findViewById(R.id.prefDialogSeekBarText);
		this.textView.setText(String.format("%1$.1f/%2$.1f", this.fValue, this.fMax));
		this.mDialog = dialog;

		dialog.show();
	}

	public void onClick(View v) {
		this.mDialog.dismiss();
		Preferences.setFloat(super.getKey(), this.fValue);
	}

	public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
		this.fValue = progress / this.fDivider + this.fMin;
		this.textView.setText(String.format("%1$.1f/%2$.1f", this.fValue, this.fMax));
	}

	public void onStartTrackingTouch(SeekBar seekBar) {
		// Not use
	}

	public void onStopTrackingTouch(SeekBar seekBar) {
		// Not use
	}

}
